//-----------------------------------------------------------------
// SX15-Evo
// Firmware di test per uMMC
// Data : 25-01-06
// Versione : 1.0
// Area SX
// http://www.areasx.com
// Note     :
//
// TEST MMC -> DIP_S3 2:ON 4:ON  Seriale D
// Se si vuole  possibile utilizzare anche la seriale B impostando
//             DIP_S3 6:ON 8:ON  Seriale B
//
//-----------------------------------------------------------------


//-----------------------------------------------------------------
// Define Hardware
//-----------------------------------------------------------------
#define LEDLD1_ON       BitWrPortI(PFDR,&PFDRShadow,1,1)
#define LEDLD1_OFF      BitWrPortI(PFDR,&PFDRShadow,0,1)
#define LEDLD2_ON       BitWrPortI(PFDR,&PFDRShadow,1,0)
#define LEDLD2_OFF      BitWrPortI(PFDR,&PFDRShadow,0,0)
#define LEDLD3_ON       BitWrPortI(PFDR,&PFDRShadow,1,4)
#define LEDLD3_OFF      BitWrPortI(PFDR,&PFDRShadow,0,4)
#define BUTTON_PRESS    BitRdPortI(PEDR,1)

//-----------------------------------------------------------------
// Define SERIALI
//-----------------------------------------------------------------
#define DINBUFSIZE  511
#define DOUTBUFSIZE  511


// -----------------------------------------------------------------------
// Variabili globali
// -----------------------------------------------------------------------
unsigned long timer;

char txBuff[DOUTBUFSIZE];
char rxBuff[DINBUFSIZE];

//-----------------------------------------------------------------
// Prototipi
//-----------------------------------------------------------------
void BoardInit(void);
void msDelay(unsigned int);
int CommandUMMC(char *Cmd, char *ReceivedData, int WaitChar);


//--------------------------------------------------------
// Ritardo bloccante
// Riceve      : ams - numero di millisecondi da ritardare
// Restituisce : -
// Note        :
//--------------------------------------------------------
nodebug
void msDelay(unsigned int ams)
{
  auto unsigned long t0;
  for(t0=MS_TIMER; MS_TIMER<ams+t0;);
}

//--------------------------------------------------------
// Inizializza i port del modulo RCM
// Riceve      : -
// Restituisce : -
// Note        :
//--------------------------------------------------------
void BoardInit() {

  // Port A
  // Default ingresso

  // Port B
  // Default

  // Port C
  // PC0-PC1 - Seriale D
  BitWrPortI(PCFR, &PCFRShadow, 1,0);

  // PC2-PC3 - Seriale C
  // Modem GSM
  BitWrPortI(PCFR, &PCFRShadow, 1,2);

  // Port D
  // PD4-PD5 - Seriale B

  // Port E
  WrPortI(PEFR, &PEFRShadow, 0x00);
  // PE0 - DCD Modem GSM (ingresso)
  // PE1 - Pulsante esterno (ingresso)
  // PE4 - GSM Power Control (ingresso)
  // PE5 - RS485 Data Direction / RS232 DTR (output)
  BitWrPortI(PEDDR, &PEDDRShadow, 1,5);
  // PE7 - GSM On/Off (output)
  BitWrPortI(PEDDR, &PEDDRShadow, 1,7);

  // Port F
  WrPortI(PFFR, &PFFRShadow, 0x00);
  WrPortI(PFDCR, &PFDCRShadow, 0x00);
  // PF0 - Led LD3 (output)
  BitWrPortI(PFDDR, &PFDDRShadow, 1,0);
  // PF1 - Led LD1 (output)
  BitWrPortI(PFDDR, &PFDDRShadow, 1,1);
  // PF4 - Led LD2 (output)
  BitWrPortI(PFDDR, &PFDDRShadow, 1,4);
  // PF5 - Modem Reset (output)
  BitWrPortI(PFDDR, &PFDDRShadow, 1,5);
  // PF6 - SH71 External I2c (bid)
  // PF7 - SH71 External I2c (bid)

  // Port G
  WrPortI(PGFR, &PGFRShadow, 0x00);
  // PG6-PG7 - Seriale E (RF Transmitter, Caller Id, GPS Receiver)
  BitWrPortI(PGFR, &PGFRShadow, 1,6);
  BitWrPortI(PGDDR, &PGDDRShadow, 0,7);
  BitWrPortI(PGDDR, &PGDDRShadow, 1,6);
}

//--------------------------------------------------------
// Invia comandi al modulo uMMC sulla seriale D
// Riceve      : Cmd Comando da inviare
//               ReceivedData  buffer per la risposta dal modulo
//               int WaitChar numero di caratteri che attendono
// Restituisce : intero 1 operazione riuscita
//               -1 errore
//               0 nessuna risposta
// Note        : 
//--------------------------------------------------------
int CommandUMMC(char *Cmd, char *ReceivedData, int WaitChar){
  auto int i;
  auto unsigned long t0;

  // Invio del comando
  serDputs(Cmd);
  
  t0=MS_TIMER;
  while(serDpeek()==-1 &&  MS_TIMER<1000+t0);
  if ((i = serDread(ReceivedData, WaitChar, 500)) > 0) {
    ReceivedData[i]=0;
    
    if(ReceivedData[0]!='E') {
      //Risposta senza errori
      return (1);
    } else {
      //Il modulo uMMC ha restituito un errore
      return (-1);
    }
  } else {
    //Nessuna risposta dal dispositivo
    return(0);
  }

}



// -----------------------------------------------------------------------
// -----------------------------------------------------------------------
//                               MAIN
// -----------------------------------------------------------------------
// -----------------------------------------------------------------------
main () {
  int i;
  char tmp[255];
  struct tm rtc;
  char ch;
  char cmd;
  char s[100];


  // Inizializzazione Scheda
  BoardInit();

  // Apertura della porta seriale D
  serDopen(9600);

  printf ("SX15-Evo: System Ready\n");
  printf ("Attesa per il controllo della MMC (puo' richiedere fino a 30 secondi)\n\n");

  
  printf ("Tempo rimasto:  ");
  timer=SEC_TIMER;
  while(serDpeek()==-1 &&  SEC_TIMER<30+timer) {
    printf ("\b\b%02d",30-(SEC_TIMER-timer));    
  }
  printf ("\n\n");

  i=serDread(rxBuff,10,100);

  if (rxBuff[0]='>') {
    printf ("-> Memory card pronta\n");
  } else {
    if (!i) {
      printf ("-> Nessuna comunicazione dalla nemory card\n");
    } else {
      printf ("-> Risposta sconosciuta dalla memory card\n");
    }
  }


  // Ricava versione e serial number
  i=CommandUMMC("V\r", rxBuff, 25);
  if(i==-1) {
    printf("-> ERRORE - codice: %s\n", rxBuff);
    exit(-1);
  } else if(i==0) {
    printf("-> Nessuna risposta dal modulo uMMC\n");
    exit(-1);
  }
  printf("-> uMMC Versione [%s]\n", rxBuff);

  while(1) {

    // Lampeggio di un led
    costate {
      LEDLD1_ON;
      DelayMs(400);
      LEDLD1_OFF;
      DelayMs(400);
    }

    costate {

		  printf("\nInserisci il comando desiderato:\n ");

      printf("\n1 - Ricava la dimensione della memory card ");
      printf("\n2 - Scrivi una linea sul file di test (rabbit.txt) ");
      printf("\n3 - Leggi l'inizio del file di test");
      printf("\n4 - Cancella il file di test");

		  printf("\nInserisci il comando da inviare: ");
		  gets(s);
		  cmd = atoi(s);

      switch (cmd) {

      case 1:

        //Invia comando per sapere le dimensioni della memory card inserita
        i=CommandUMMC("Q\r", rxBuff, 14);
        if(i==-1) {
          printf("-> ERRORE - codice: %s\n", rxBuff);
          break;
        } else if(i==0) {
          printf("-> Nessuna risposta dal modulo uMMC\n");
          break;
        }

        printf("-> Spazio sulla memoria %s\n", rxBuff);
        break;

      case 2:
        //Apre il file rabbit.txt se non esiste lo crea
        i=CommandUMMC("O 1 A /rabbit.txt\r", rxBuff, 4);
        if(i==-1) {
          printf("-> ERRORE - codice: %s\n", rxBuff);
          break;;
        }
        printf("-> Creazione o apertura del file rabbit.txt riuscita\n");

        // Ricavo ora e data correnti dall'RTC
        mktm(&rtc, read_rtc());

        // Preparo il testo da inserire nel file
        sprintf(tmp, "%02d/%02d/%04d %02d:%02d:%02d il rabbit scrive :)\r\n", rtc.tm_mon, rtc.tm_mday, 1900+rtc.tm_year, rtc.tm_hour, rtc.tm_min, rtc.tm_sec);
        printf(tmp);

        // Scrivo nel file
        printf("Scrittura nel file....\n");
        sprintf(txBuff, "W 1 %d\r%s", strlen(tmp), tmp);
        i=CommandUMMC(txBuff, rxBuff, 4);
        if(i==-1) {
          printf("-> ERRORE - codice: %s\n", rxBuff);
          break;
        }

        printf("-> Scrittura terminata\n");

        //Chiude il file
        printf("Chiusura del file....\n");
        i=CommandUMMC("C 1\r", rxBuff, 4);
        if(i==-1) {
          printf("-> ERRORE - codice: %s\n", rxBuff);
          break;
        }
        printf("-> File chiuso\n");
        break;

      case 3:
        //Apre il file rabbit.txt se non esiste lo crea
        i=CommandUMMC("O 1 R /rabbit.txt\r", rxBuff, 4);
        if(i==-1) {
          printf("-> ERRORE - codice: %s\n", rxBuff);
          break;
        }
        printf("-> Apertura del file rabbit.txt riuscita\n");

        // Leggo dal file
        printf("Lettura di 400 byte dal file....\n");
        sprintf(txBuff, "R 1 400\r");
        i=CommandUMMC(txBuff, rxBuff, 400);
        if(i==-1) {
          printf("-> ERRORE - codice: %s\n", rxBuff);
          break;
        }

        printf("-> Lettura terminata\n");
        printf ("\nDati letti dal file:\n");
        printf ("[%s]\n\n",rxBuff);        

        //Chiude il file
        printf("Chiusura del file....\n");
        i=CommandUMMC("C 1\r", rxBuff, 4);
        if(i==-1) {
          printf("-> ERRORE - codice: %s\n", rxBuff);
          break;
        }
        printf("-> File chiuso\n");



        break;

      case 4:
        //Apre il file rabbit.txt se non esiste lo crea
        i=CommandUMMC("E /rabbit.txt\r", rxBuff, 4);
        if(i==-1) {
          printf("-> ERRORE - codice: %s\n", rxBuff);
          break;
        }
        printf("-> Cancellazione del file rabbit.txt riuscita\n\n");
        break;

      default:
        printf ("-> Comando sconosciuto\n");
        break;

      }

    }

  }


}

